# cython: language_level=3

# Copyright (c) 2014, Dr Alex Meakins, Raysect Project
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     1. Redistributions of source code must retain the above copyright notice,
#        this list of conditions and the following disclaimer.
#
#     2. Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#
#     3. Neither the name of the Raysect Project nor the names of its
#        contributors may be used to endorse or promote products derived from
#        this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from raysect.core.math._vec3 cimport _Vec3
from raysect.core.math.vector cimport Vector
from raysect.core.math.affinematrix cimport AffineMatrix

cdef class Normal(_Vec3):

    cpdef Vector cross(self, _Vec3 v)

    cpdef Normal normalise(self)

    cpdef Normal transform(self, AffineMatrix m)

    cpdef Normal transform_with_inverse(self, AffineMatrix m)

    cdef inline Normal neg(self)

    cdef inline Normal add(self, _Vec3 v)

    cdef inline Normal sub(self, _Vec3 v)

    cdef inline Normal mul(self, double m)

    cdef inline Normal div(self, double m)

    cpdef Normal copy(self)

    cpdef Vector as_vector(self)

    cpdef Vector orthogonal(self)


cdef inline Normal new_normal(double x, double y, double z):
    """
    Normal factory function.

    Creates a new Normal object with less overhead than the equivalent Python
    call. This function is callable from cython only.
    """

    cdef Normal v
    v = Normal.__new__(Normal)
    v.x = x
    v.y = y
    v.z = z
    return v
